﻿/// <reference path="_references.js" />

(function (window, undefined) {
    // Define the "MyApp" namespace
    var MyApp = window.MyApp = window.MyApp || {};

    // Person class
    var entityType = "Person:#SamplePortableClassLibrary.Models";
    MyApp.Person = function (data) {
        var self = this;

        // Underlying data
        self.Id = ko.observable(data.Id);
        self.Age = ko.observable(data.Age);
        self.Name = ko.observable(data.Name);
        self.Sex = ko.observable(data.Sex);
        upshot.addEntityProperties(self, entityType);
    }

    // PeopleViewModel class
    MyApp.PeopleViewModel = function (options) {
        var self = this;

        // Private properties
        var dataSourceOptions = {
            providerParameters: { url: options.serviceUrl, operationName: "GetPeople" },
            entityType: entityType,
            bufferChanges: true,
            mapping: MyApp.Person
        };
        var gridDataSource = new upshot.RemoteDataSource(dataSourceOptions);
        var editorDataSource = new upshot.RemoteDataSource(dataSourceOptions);

        // Data
        self.people = gridDataSource.getEntities();
        self.editingPerson = editorDataSource.getFirstEntity();
        self.successMessage = ko.observable().extend({ notify: "always" });
        self.errorMessage = ko.observable().extend({ notify: "always" });
        self.paging = new upshot.PagingModel(gridDataSource, {
            onPageChange: function (pageIndex, pageSize) {
                self.nav.navigate({ page: pageIndex, pageSize: pageSize });
            }
        });
        self.validationConfig = $.extend({
            resetFormOnChange: self.editingPerson,
            submitHandler: function () { self.saveEdit() }
        }, editorDataSource.getEntityValidationRules());

        // Client-side navigation
        self.nav = new NavHistory({
            params: { edit: null, page: 1, pageSize: 10 },
            onNavigate: function (navEntry, navInfo) {
                self.paging.moveTo(navEntry.params.page, navEntry.params.pageSize);

                // Wipe out any old data so that it is not displayed in the UI while new data is being loaded 
                editorDataSource.revertChanges();
                editorDataSource.reset();

                if (navEntry.params.edit) {
                    
                    if (navEntry.params.edit == "new") {
                        // Create and begin editing a new person instance
                        editorDataSource.getEntities().push(new MyApp.Person({}));
                    } else {
                        // Load and begin editing an existing person instance
                        editorDataSource.setFilter({ property: "Id", value: Number(navEntry.params.edit) }).refresh();
                    }
                } else {
                    // Not editing, so load the requested page of data to display in the grid
                    gridDataSource.refresh();
                }
            }
        }).initialize({ linkToUrl: true });

        // Public operations
        self.saveEdit = function () {
            editorDataSource.commitChanges(function () {
                self.successMessage("Saved Person").errorMessage("");
                self.showGrid();
            });
        }
        self.revertEdit = function () { editorDataSource.revertChanges() }
        self.editPerson = function (person) { self.nav.navigate({ edit: person.Id() }) }
        self.showGrid = function () { self.nav.navigate({ edit: null }) }
        self.createPerson = function () { self.nav.navigate({ edit: "new" }) }
        self.deletePerson = function (person) {
            editorDataSource.deleteEntity(person);
            editorDataSource.commitChanges(function () {
                self.successMessage("Deleted Person").errorMessage("");
                self.showGrid();
            });
        };

        // Error handling
        var handleServerError = function (httpStatus, message) {
            if (httpStatus === 200) {
                // Application domain error (e.g., validation error)
                self.errorMessage(message).successMessage("");
            } else {
                // System error (e.g., unhandled exception)
                self.errorMessage("Server error: HTTP status code: " + httpStatus + ", message: " + message).successMessage("");
            }
        }
        
        gridDataSource.bind({ refreshError: handleServerError, commitError: handleServerError });
        editorDataSource.bind({ refreshError: handleServerError, commitError: handleServerError });
    }
})(window);

